/**
 * NeoClock
 * 
 * Clock using 60 WS2812B/Neopixel LEDs and DS3231 RTC
 * 
 * Libraries needed:
 *   * Adafruit NeoPixel (Library Manager) - Phil Burgess / Paint Your Dragon for Adafruit Industries - LGPL3
 *   * 
 *   * Arduino Timezone Library (https://github.com/JChristensen/Timezone) - Jack Christensen - CC-BY-SA
 *   * Time Library (https://github.com/PaulStoffregen/Time) - Paul Stoffregen, Michael Margolis - LGPL2.1
 *   
 *   Added intensity driver wit an LDR on an analogue input A0!
 *   
 *   
 *   
 *   
 *   
 *   
 **/
int shape=42;//41=LEFT_TOP;42=RIGHT_TOP;43=LEFT_BOTTOM;44=RIGHT_BOTTOM
int ledtype=2; // 1=BIG LEDS , 2=small leds 4 segments for 60 led clock (the small leds need less energy)
int animation=1; // animation on=1; off=0
// 1=star with BOTTOM dual feed and single LED in the tips like the 60 leds steel framed star
//11=star with BOTTOM dual feed and dual LED in the tips like the 40 leds ministar
//12=star with BOTTOM dual feed and dual LED in the tips especially written for the 150 leds bigstar, mostly alike nr 11
// 2=star with TOP single feed and single LED in the tips 
// 3=star with TOP dual feed and dual LED in the tips
// 4=Ring 
// 41=clock top feed with leds placed in left direction
// 42=clock top feed with leds placed in right direction
// 43=clock bottom feed with leds placed in left direction
// 44=clock bottom feed with leds placed in right direction
// 5=Bottom feed star with 102 leds
// 6=Single string of leds


#define NUM_LEDS 60
#include <FastLED.h>
#define LED_TYPE   WS2811
#define COLOR_ORDER   GRB  // for old-fashioned chinese WS2812(B)/ WS2811+5050GRB led strips 
#define DATA_PIN        5
const int ldrPin = A0; // analog pin 0, connected with photovaristor
int ldrStatus=50;// initialize variable val for light intensity variable infuencing
int intensity=25;
#define BRIGHTNESS   56
int ghue = 0; 
CRGBArray<NUM_LEDS> leds;

#ifdef __AVR__
  #include <avr/power.h>
#endif

#if defined(ESP8266)
#include <pgmspace.h>
#else
#include <avr/pgmspace.h>
#endif

#include <Wire.h> // must be included here so that Arduino library object file references work
#include <RtcDS3231.h>// Makuna library
RtcDS3231<TwoWire> Rtc(Wire);
/* for normal hardware wire use above */

int (TIMING) = 6; // multiplication factor for the wait variable, if so needed was 2; higher = slower!!
int (mintiming)=6;//fastest
int (maxtiming)=8;//slowest
int (wait_multiplier)=1;
int (dimmernormal)=50;
int (mindimmer)=30;//lowest intensity
int (maxdimmer)=155;//brightest intensity
int (ij)=0;
int (ii)=0;
boolean fader=true;

#include <TimeLib.h>        //http://www.arduino.cc/playground/Code/Time
#include <Timezone.h>    //https://github.com/JChristensen/Timezone
#include <EEPROM.h>

//Central European Time (Frankfurt, Paris)
TimeChangeRule CEST = {"CEST", Last, Sun, Mar, 2, 120};     //Central European Summer Time
TimeChangeRule CET = {"CET ", Last, Sun, Oct, 3, 60};       //Central European Standard Time
Timezone CE(CEST, CET);
TimeChangeRule *tcr;        //pointer to the time change rule, use to get the TZ abbrev
time_t utc;

unsigned long lastMillis = millis();
byte dimmer = 0x88;
byte hmark = 0;
byte ohour=0;
byte ominute=0;
byte osecond=0;
////////////////////////////////////////////////////////////////////////////////////////
void setup() {

FastLED.addLeds<WS2811,DATA_PIN,GRB>(leds,NUM_LEDS).setCorrection(TypicalLEDStrip);

// set master brightness control
  FastLED.setBrightness(BRIGHTNESS);
   
  Serial.begin(57600);
  pinMode(ldrPin, INPUT_PULLUP); // Here the LDR sensor is determined as input with an internal pull-up resistor.
  autodim();

Rtc.Begin();

  Rtc.Enable32kHzPin(false);
  Rtc.SetSquareWavePin(DS3231SquareWavePin_ModeNone); 

  if (!Rtc.GetIsRunning())
  {
      Serial.println("Rtc was not actively running, starting now");
      Rtc.SetIsRunning(true);
  }

  if (!Rtc.IsDateTimeValid()) 
  {
      // Common Cuases:
      //    1) the battery on the device is low or even missing and the power line was disconnected
      Serial.println("Rtc lost confidence in the DateTime!");
  }

  byte eechk = EEPROM.read(0);
  if(eechk == 0xAA) { //Assume this is our config and not a fresh chip
    dimmer = EEPROM.read(1);
    hmark = EEPROM.read(2);
    fader = EEPROM.read(3);
  }

  timeSync();
FastLED.clear();  // clear all pixel data

}// end of setup
///////////////////////////////////////////////////////////////////
void addPixelColor(byte pixel, byte color, byte brightness) {
// 41=clock top feed with leds set in the left direction
// 42=clock top feed with leds set in the right direction
// 43=clock bottom feed with leds set in the left direction
// 44=clock bottom feed with leds set in the right direction



if (shape==43||shape==44){pixel=pixel+30; if(pixel>=NUM_LEDS){pixel=pixel-NUM_LEDS;}}// BOTTOM FEED

//pixel=pixel*(NUM_LEDS/60);// can be handy when using more than 60 leds for a clock

  color *= 8;// was 8
 // uint8_t acolor = brightness;
 uint8_t acolor = color;
  acolor <<= color;
 
      if (shape==41||shape==43){//LEFT DIRECTION LEDS
      uint8_t ocolor = leds[(NUM_LEDS)-1-pixel].getLuma();
//      uint8_t avgLight = leds[(NUM_LEDS)-1-pixel].getAverageLight();
      ocolor |= acolor;
      leds[(NUM_LEDS)-1-pixel] = CHSV(ocolor, 255, brightness); // hue, saturation 0-255, brightness 0-255
      }
      
      if (shape==42||shape==44){//RIGHT DIRECTION LEDS
      uint8_t ocolor = leds[pixel].getLuma();
//      uint8_t avgLight = leds[pixel].getAverageLight();
      ocolor |= acolor;
      leds[pixel] = CHSV (ocolor, 255, brightness); // hue, saturation 0-255, brightness 0-255
      
      }
}

          //FASTLED EXAMPLES:
          
          //leds[i] = CHSV(a, 255, 255); // hue, saturation 0-255, brightness 0-255
          
          // Set up a CHSV color
           // CHSV paleBlue( 160, 128, 255); hue, saturation, brightness 0-255
          
            // Now...
            //   paleBlue.hue == 160
            //   paleBlue.sat == 128
            //   paleBlue.val == 255
          
          // HSV (Rainbow) to RGB color conversion
          //  CHSV hsv( 160, 255, 255); // pure blue in HSV Rainbow space
          //  CRGB rgb;
          //  hsv2rgb_rainbow( hsv, rgb);
            // rgb will now be (0, 0, 255)  -- pure blue as RGB
          
          // Add one CRGB color to another.
          //  leds[i] += CRGB( 20, 0, 0);
          
            // Add a constant amount of brightness to all three (RGB) channels.
          //  leds[i].addToRGB(20);
          
            // Add a constant "1" to the brightness of all three (RGB) channels.
          //  leds[i]++;
          
          
            // Subtract one color from another.
          //  leds[i] -= CRGB( 20, 0, 0);
          
            // Subtract a contsant amount of brightness from all three (RGB) channels.
          //  leds[i].subtractFromRGB(20);
          
            // Subtract a constant "1" from the brightness of all three (RGB) channels.
          //  leds[i]--;
          
          
           // Get brightness, or luma (brightness, adjusted for eye's sensitivity to
            // different light colors.   See http://en.wikipedia.org/wiki/Luma_(video) )
          //  uint8_t luma = leds[i].getLuma();
          //  uint8_t avgLight = leds[i].getAverageLight();
          // Set color from Hue, Saturation, and Value.  
            // Conversion to RGB is automatic.
          //  leds[i] = CHSV( 160, 255, 255);
          
            // alternate syntax
          //  leds[i].setHSV( 160, 255, 255);
          
            // set color to a pure, bright, fully saturated, hue
          //  leds[i].setHue( 160);

///////////////////////////////////////////////////////////////////
void calcTime(void) {
  utc = now();
  CE.toLocal(utc, &tcr);
  ohour = hour(utc);
  ominute = minute(utc);
  if(osecond != second(utc)) {
    osecond = second(utc);
    lastMillis = millis();

    if(ominute == 0 && osecond == 0) {
      //Every hour
      timeSync();
    }
  }
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void drawClock(byte h, byte m, byte s) {  
   FastLED.clear();  // clear all pixel data
   FastLED.setBrightness(155);
   
// Add blue low intensity dots for 12(0),3, 6 and 9 O'çlock to verify where the clock is positioned.
for(int i = 0; i<4; i++) { addPixelColor((15*i), 120, mindimmer/2); }  

   
addPixelColor(m, 140, maxdimmer);// Minutes in GREEN// was maxdimmer+100

 
  if(hmark > 0) {
  for(byte i = 0; i<12; i++) {
      addPixelColor((5*i), 2, hmark);
    }
  }

  h %= 12;
  h *= 5;
  h += (m/12);
  addPixelColor(h, 160, maxdimmer);// Hours in RED// was maxdimmer+100
  
 if (osecond == 59 && animation==1){icicle (6,1); }
  
  // 0x RR GG BB

  if(fader) {
    byte dim_s1 = maxdimmer;
    byte dim_s2 = 0;
    byte px_s2 = s+1;
    if(px_s2 >= 60) px_s2 = 0;
    unsigned long curMillis = millis()-lastMillis;
    if(curMillis < 250) {
      dim_s2 = 20;
      dim_s1 = 250;
    }else{
      dim_s2 = map(curMillis, 250, 1000, 0, 250);
      dim_s1 = 250 - map(curMillis, 250, 1000, 0, 250);
    }
    
    addPixelColor(s, 120, dim_s1/2);//(byte pixel, byte color, byte brightness)
    addPixelColor(px_s2, 120, dim_s2/2);
  }else{
    addPixelColor(s, 120, mindimmer/2);// was maxdimmer + 100
  }

  FastLED.show(); 
  
}
byte rounds = 0;
////////////////////////////////////////////////////////////////////////////////////////////
void timeSync(void) {

  RtcDateTime dt = Rtc.GetDateTime();
  setTime(dt.Hour(),dt.Minute(),dt.Second(),dt.Day(),dt.Month(),dt.Year());
    
  Serial.print("Synced to: ");
  Serial.print(dt.Year());
  Serial.print("-");
  Serial.print(dt.Month());
  Serial.print("-");
  Serial.print(dt.Day());
  Serial.print("-");
  Serial.print(dt.Hour());
  Serial.print("-");
  Serial.print(dt.Minute());
  Serial.print("-");
  Serial.println(dt.Second());
   
  Serial.print("dimmer= ");    
  Serial.print(EEPROM.read(1));
  Serial.print("; hmark= ");    
  Serial.print(EEPROM.read(2));
  Serial.print("; fader= ");    
  Serial.println(EEPROM.read(3));
  
}
///////////////////////////////////////////////////////////////////////////////////
void timeSave(void) {
  utc = now();
  
  RtcDateTime store = RtcDateTime(year(utc), month(utc), day(utc), hour(utc), minute(utc), second(utc));
  Rtc.SetDateTime(store);

  Serial.print("Synced to: ");
  Serial.print(year(utc));
  Serial.print("-");
  Serial.print(month(utc));
  Serial.print("-");
  Serial.print(day(utc));
  Serial.print("-");
  Serial.print(hour(utc));
  Serial.print("-");
  Serial.print(minute(utc));
  Serial.print("-");
  Serial.println(second(utc));
  
}
///////////////////////////////////////////////////////////////////////////
void setBrightness(byte brightness) {
  //dimmer = brightness;
}

void chkSer(void) {
  unsigned int iy;
  byte im,id,iH,iM,iS;
  
  if(!Serial.available()) return;

  switch(Serial.read()) {
    case 'b':
      setBrightness(Serial.parseInt());
      //setBrightness(dimmernormal);
      
setBrightness(dimmernormal);
      
      Serial.print(F("Brightness changed to: "));
      Serial.println(dimmernormal);
      EEPROM.put(0, 0xAA);
      EEPROM.put(1, dimmer);
      break;
    case 't':
      iy = Serial.parseInt();
      im = Serial.parseInt();
      id = Serial.parseInt();
      iH = Serial.parseInt();
      iM = Serial.parseInt();
      iS = Serial.parseInt();
      setTime(iH,iM,iS,id,im,iy);
      Serial.println(F("System time changed"));
      break;
    case 'f':
      fader = false;
      EEPROM.put(0, 0xAA);
      EEPROM.put(3, 0);
      Serial.println(F("Fader off"));
      break;
    case 'F':
      fader = true;
      EEPROM.put(0, 0xAA);
      EEPROM.put(3, 1);
      Serial.println(F("Fader on"));
      break;
    case 'm':
      hmark = Serial.parseInt();
      EEPROM.put(0, 0xAA);
      EEPROM.put(2, hmark);
      Serial.println(F("HMark changed"));
      break;
    case 's':
      timeSync();
      Serial.println(F("Synced RTC to System"));
      break;
    case 'S':
      timeSave();
      Serial.println(F("Synced System to RTC"));
      break;
    default:
      Serial.println('?');
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
 void SHOWTIME() {
  calcTime();
  if(rounds++ > 1000) {
    Serial.print(ohour);
    Serial.print(":");
    Serial.print(ominute);
    Serial.print(":");
    Serial.print(osecond);
    Serial.print("(C)JG-2023, temp= ");
    RtcTemperature temp = Rtc.GetTemperature();
  //temp.Print(Serial);
  // you may also get the temperature as a float and print it
    Serial.print(temp.AsFloatDegC());
    Serial.println(" degC ");
    autodim();
    rounds = 0;
  }
  else {drawClock(ohour,ominute,osecond);}
 
  delay(10);
  chkSer();
}//END OF VOID "SHOWTIME()"
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//redefine dimmer, mindimmer and maxdimmer here, related to ldrStatus!
void autodim(){ 
ldrStatus = analogRead(ldrPin);//read the analog value of the light sensor, in darkness= approx. 1000 and brightest = approx. 10
dimmernormal=3+(850-ldrStatus)/3;// was /3
if (dimmernormal<7&&ledtype==2){dimmernormal=37;}
if (dimmernormal<27&&ledtype==1){dimmernormal=57;}
if (dimmernormal>200){dimmernormal=150;}
dimmer=dimmernormal;
mindimmer=dimmernormal;
maxdimmer=dimmernormal+50;
FastLED.setBrightness(dimmernormal);
Serial.print("ldRstatus = : ");
Serial.print(ldrStatus);
Serial.print(" ; dimmernormal = : ");
Serial.print(dimmernormal);
Serial.print(" ; mindimmer = : ");
Serial.print(mindimmer);
Serial.print(" ; maxdimmer = : ");
Serial.println(maxdimmer);
//Serial.print(" ; ocolor = : ");
//Serial.println(ocolor);
}
///////////////////////////////////////////////////////////////////////////////////
//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {FastLED.setBrightness(mindimmer/2);
  for (int j=0; j<2; j++) {  //do 4  cycles of chasing
    for (int q=0; q < 3; q++) {
      for (uint16_t i=0; i < (NUM_LEDS); i=i+3) {
      leds[(i+q)] = CHSV(c, 255, 255); //turn every third pixel on, hue, saturation 0-255, brightness 0-255
      }
      FastLED.show(); 
      delay(wait);
      for (uint16_t i=0; i < (NUM_LEDS); i=i+3) {
      leds[(i+q)] = CHSV(0, 255, 255); //turn every third pixel off,hue, saturation 0-255, brightness 0-255
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////////////////////////
// void icicle in R direction, top down for string with top feeding
void icicle (int(wait),int (times)) { Serial.print("icicle running now ");Serial.print("shape= : "); Serial.println(shape);

static int hue = 0; 
int add =(NUM_LEDS)/5;
autodim(); 
int a = 255;
int b = 255;
int c = 255;
int fadefast=232;
                               
{for(int ii = 0; ii < times; ii++) {
  
                  leds[0] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier*20); //get the iced water at the collecting point
                  leds[1] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier*15); //get the iced water at the collecting point
                  leds[2] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier*20);  //get the iced water at the collecting point 
                  leds[3] = CRGB(a/2,b/2,c/2); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier*20);   //get the iced water at the collecting point    
                  leds[0] = CRGB(a/2,b/2,c/2); FastLED.show(); fadeall(244); delay(wait*wait_multiplier*20); //get the iced water at the collecting point
                  leds[1] = CRGB(a/2,b/2,c/2); FastLED.show(); fadeall(244); delay(wait*wait_multiplier*20); //get the iced water at the collecting point
                  leds[2] = CRGB(a/2,b/2,c/2); FastLED.show(); fadeall(244); delay(wait*wait_multiplier*20);  //get the iced water at the collecting point   
                  leds[3] = CRGB(a,b,c); FastLED.show(); fadeall(244); delay(wait*wait_multiplier*10);   //get the iced water at the collecting point               
                  leds[4] = CRGB(a/2,b/2,c/2); FastLED.show(); fadeall(244); delay(wait*wait_multiplier*2);  //the collected water to be dropped                 
                
                  for(int i = 5; i <add*1; i++) {leds[i] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier*2);}//the falling drop first part slow
                  for(int i = add*1+1; i <add*1.5; i++) {leds[i] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier);}//the falling drop second part medium slow
                  for(int i = add*1.5+1; i <(NUM_LEDS)-2; i++) {leds[i] = CRGB(a/4,b/4,c/4); FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier/2);}//the falling drop last part fast               
                  
                  for(int i = 0; i <2; i++) {
                  leds[NUM_LEDS-1] = CRGB(a,b,c); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel full power
                  leds[NUM_LEDS-2] = CRGB(a/2,b/2,c/2); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel half power
                  leds[NUM_LEDS-1] = CRGB(a/2,b/2,c/2); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel full power
                  leds[NUM_LEDS-2] = CRGB(a,b,c); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel half power
                  leds[NUM_LEDS-1] = CRGB(a,b,c); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel full power
                  leds[NUM_LEDS-1] = CRGB(a/2,b/2,c/2); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel half power
                  leds[NUM_LEDS-1] = CRGB(a,b,c); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel full power
                  leds[NUM_LEDS-1] = CRGB(a/2,b/2,c/2); FastLED.show(); delay(wait*wait_multiplier/2); //show the last pixel half power
                  }
                  
                  for(int i = NUM_LEDS+10; i >0; i--) {FastLED.show(); fadeall(fadefast); delay(wait*wait_multiplier);}// fade all pixels

                
    delay(1000);                
                }}  


   }   
//////////////////////////////////////////////////////////////////////////////////////////////////
void fadeall(int (fade)) { for(int i = 0; i < NUM_LEDS; i++) { leds[i].nscale8(fade); } }   
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////    
void loop(){
SHOWTIME();
}//end of loop
//////////////////////////////////////////////////////////////////////////////////////////////////////
 
